# Copyright 2026 Flyto2. Licensed under Apache-2.0. See LICENSE.

"""
Browser Composite Modules

High-level browser automation workflows combining multiple atomic modules.
"""
from .scrape_to_json import WebScrapeToJson

__all__ = [
    'WebScrapeToJson',
]
